grammar Mu;

prog
 : block EOF
 ;

block
 : stat*   #statList
 ;

stat
 : assignment
 | if_stat
 | while_stat
 | log   
 | OTHER {System.err.println("unknown char: " + $OTHER.text);}
 ;

assignment
 : ID ASSIGN expr SCOL #assignStat
 ;

if_stat
 : IF condition_block (ELSE IF condition_block)* (ELSE stat_block)? #ifStat
 ;

condition_block
 : expr stat_block  #condBlock
 ;

stat_block
 : OBRACE block CBRACE
 | stat
 ;

while_stat
 : WHILE expr stat_block #whileStat
 ;

log
 : LOG expr SCOL #logStat
 ;

expr
 : <assoc=right> expr POW expr          #powExpr
 | MINUS expr                           #unaryMinusExpr
 | NOT expr                             #notExpr
 | expr opmul  expr           #multiplicationExpr
 | expr oplus expr          #additiveExpr
 | expr oprel  expr          #relationalExpr
 | expr opeq expr              #equalityExpr
 | expr AND expr                        #andExpr
 | expr OR expr                         #orExpr
 | atom                                 #atomExpr
 ;

atom
 : OPAR expr CPAR #parExpr
 | (INT | FLOAT)  #numberAtom
 | (TRUE | FALSE) #booleanAtom
 | ID             #idAtom
 | STRING         #stringAtom
 | NIL            #nilAtom
 ;

opmul:  
	MULT #opMul
    | DIV    #opMul
    | MOD   #opMul
;

oplus: PLUS #opPlus
    | MINUS #opPlus
    ;

oprel:
	GT #opRel
    | LT   #opRel
    | GTEQ  #opRel
    | LTEQ #opRel
    ;
opeq: EQ #opEq
    | NEQ #opEq
    ;
	
OR : '||';
AND : '&&';
EQ : '==';
NEQ : '!=';
GT : '>';
LT : '<';
GTEQ : '>=';
LTEQ : '<=';
PLUS : '+';
MINUS : '-';
MULT : '*';
DIV : '/';
MOD : '%';
POW : '^';
NOT : '!';

SCOL : ';';
ASSIGN : '=';
OPAR : '(';
CPAR : ')';
OBRACE : '{';
CBRACE : '}';

TRUE : 'true';
FALSE : 'false';
NIL : 'nil';
IF : 'if';
ELSE : 'else';
WHILE : 'while';
LOG : 'log';

ID
 : [a-zA-Z_] [a-zA-Z_0-9]*
 ;

INT
 : [0-9]+
 ;

FLOAT
 : [0-9]+ '.' [0-9]* 
 | '.' [0-9]+
 ;

STRING
 : '"' (~["\r\n] | '""')* '"'
 ;

COMMENT
 : '#' ~[\r\n]* -> skip
 ;

SPACE
 : [ \t\r\n] -> skip
 ;

OTHER
 : . 
 ;
