from MuVisitor import MuVisitor
from MuParser import *

# Implementation of Mu semantics 

class MyMuVisitor(MuVisitor):

    def __init__(self):
        self._memory=dict()

    # visitors for atoms --> value (already done!)

    def visitParExpr(self,ctx):
        return self.visit(ctx.expr())

    def visitNumberAtom(self,ctx):
        s = ctx.getText();
        try:
            return int(s)
        except ValueError:
            return float(s)

    def visitBooleanAtom(self, ctx):
        return ctx.getText() == "true"

    def visitIdAtom(self,ctx):
        try:
            return self._memory[ctx.getText()]
        except KeyError:
            raise SyntaxError("Unknown identifier %s" % ctx.getText())

    def visitStringAtom(self,ctx):
        return ctx.getText()[1:-1]

    def visitNilAtom(self,ctx): 
        return ctx.getText();

    #visit expr and eval !! 

    def visitAtomExpr(self,ctx):
        return self.visit(ctx.atom())

    def visitOrExpr(self,ctx):
        lval = self.visit(ctx.expr(0))
        rval = self.visit(ctx.expr(1))
        return lval | rval

    def visitAndExpr(self,ctx):
        lval = self.visit(ctx.expr(0))
        rval = self.visit(ctx.expr(1))
        return lval & rval

    def visitEqualityExpr(self,ctx):
        lval = self.visit(ctx.expr(0))
        rval = self.visit(ctx.expr(1))
        op = self.visit(ctx.opeq())
        #be careful for float equality
        if op == '==': #todo recup val token
            return lval == rval
        elif op == '!=':
            return lval != rval
        else:
            raise SyntaxError("Unknown comparison operator '%s'" % op)

    def visitRelationalExpr(self,ctx):
        lval= self.visit(ctx.expr(0))
        rval= self.visit(ctx.expr(1))
        uop = self.visit(ctx.oprel())
        #be careful for float equality
        if uop == '<': 
            return lval < rval
        elif (uop == '<='):
            return lval <= rval
        elif uop == '>':
            return lval > rval
        elif uop == '>=':
            return lval>=rval
        else:
            raise SyntaxError("Unknown comparison operator '%s'" % uop)

    def visitAdditiveExpr(self,ctx):
        lval= self.visit(ctx.expr(0))
        rval= self.visit(ctx.expr(1))
        op = self.visit(ctx.oplus()) 
        #be careful for float equality
        if op == '+':
            #there should be a type check here
            if any(isinstance(x, unicode) for x in (lval, rval)):
                return '{}{}'.format(lval, rval)
            else:
                return lval + rval
        elif op == '-':
            return lval - rval
        else:
            raise SyntaxError("Unknown additive operator '%s'" % op)


    def visitMultiplicationExpr(self, ctx):
        lval= self.visit(ctx.expr(0))
        rval= self.visit(ctx.expr(1))
        op = self.visit(ctx.opmul())
        #be careful for float equality
        if op == '*':
            return lval * rval #todo, type!
        elif op == '/':
            return lval / rval
        elif op == '%':
            return lval % rval
        else:
            raise SyntaxError("Unknown multiplication operator '%s'" % op)

    def visitnotExpr(self,ctx):
        return not self.visit(ctx.expr())

    def visitUnaryMinusExpr(selt,ctx):
        return -self.visit(ctx.expr())

    def visitPowEpr(self,ctx):
        lval= self.visit(ctx.expr(0))
        rval= self.visit(ctx.expr(1))
        return lval ** rval

    def visitOpPlus(self,ctx):
        return ctx.getText();

    def visitOpMul(self,ctx):
        return ctx.getText();

    def visitOpRel(self,ctx):
        return ctx.getText();
    
    def visitOpEq(self,ctx):
        return ctx.getText();

    # statements - TODO !!

    def visitLogStat(self,ctx):
        print(self.visit(ctx.expr()))

    def visitAssignStat(self,ctx): #implement!
        pass

    def visitCondBlock(self,ctx): #implement!
        pass

    def visitIfStat(self,ctx):#implement!
        pass
    
    def visitWhileStat(self,ctx):#implement!
        pass

    def visitStatList(self,ctx):
        for stat in ctx.stat():
            self.visit(stat)
